==> k8048-1.00.tar.gz: k8048/k8048/io.c <==
/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2007 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: io.c,v 1.49 2008/03/01 11:03:08 darron Exp $
 *
 * The Velleman programmer board has the following serial interface
 * characteristics (d-sub 9):
 *
 * out:
 * 
 * DSUB-9                       ICSP
 * 7        RTS -> PGC          5
 * 4        DTR -> PGD          4
 * 3        TX  -> !MCLR/VPP    1
 * 
 * in:
 *
 * 8        CTS <- PGD          4
 *
 * other:
 *
 * 9<->3    RI <- TX
 *
 * Although the interface connects to the serial port, it doesn't
 * utilise a standard serial protocol and I/O is performed using
 * the "bit banging" method.
 *
 * Data output bits are sent through the DTR line and clocked on the falling
 * edge of RTS.
 * 
 * The Tx line is utilised to enable the application of the high
 * programming voltage on !MCLR/VPP and not for sending data bits.
 *
 * Data input bits are read from the CTS line using the same clock i/f as
 * above with the PGD output taken high for correct circuit operation.
 *
 * D-SUB9
 *    1 2 3 4 5
 *     6 7 8 9
 *    1:CD
 *    2:Rx
 *    3:Tx
 *    4:DTR
 *    5:GND
 *    6:DSR
 *    7:RTS
 *    8:CTS
 *    9:RI
 *
 *
 * ICSP
 *    5 4 3 2 1
 *    5:PGC
 *    4:PGD
 *    3:VSS/GND
 *    2:VDD (+5)
 *    1:VPP (+12)
 *
 *
 * SERIAL PORT VOLTAGES
 *  
 * TEST 1
 * ------
 *  switch set to RUN, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 5V
 *       2 5V
 *       3 0V
 *       4 5V
 *       5 5V
 *
 * TEST 2
 * ------
 *  switch set to PROG, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 12V
 *       2 5V
 *       3 0V
 *       4 5V
 *       5 5V
 *
 * TEST 3
 * ------
 *  switch set to STANDBY, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 0V
 *       2 0V
 *       3 0V
 *       4 5V
 *       5 5V
 */

#include "k8048.h"

/******************************************************************************
 * debug CTRL-C detection
 *****************************************************************************/
int debug_stop=0;

/******************************************************************************
 * INIT/CLOSE I/O
 *****************************************************************************/
int
io_open(struct k8048 *k)
{
    int fd;
    
    if( (fd= open_port(k->device)) >= 0)
        io_standby(k);

    return fd;
}

void
io_close(struct k8048 *k)
{
    io_standby(k);
    close(k->fd);
}

/******************************************************************************
 * K8048 BIT BANGING NB. RS232 output is positive when feature enabled
 *****************************************************************************/

/*
 * Sleep for 1 microsecond holding output state using legacy I/O
 */
void
io_sleep(struct k8048 *k)
{
    get_cts(k->fd);
}

/*
 * Sleep for 1 microsecond holding output state using usleep
 */
void
io_usleep(struct k8048 *k)
{
    usleep(1);
}

/*
 * Set PGD (DATA)
 */
void
io_set_pgd(struct k8048 *k, int pgd)
{
    if(k->flags & FLIPDTR)      /* inverting buffer present on hardware (k8048) */
        pgd= HIGH-pgd;

    set_dtr(k->fd, pgd);        /* non inverting buffer present on hardware (k8076 *) */
                                /* * in this case the accumulation of inverters is non inverting */
}

/*
 * Set PGC (CLOCK)
 */
void
io_set_pgc(struct k8048 *k, int pgc)
{
    if(k->flags & FLIPRTS)
        pgc= HIGH-pgc;

    set_rts(k->fd, pgc);
}

/*
 * Set PGD/PGC (DATA/CLOCK) 
 */
void
io_set_pgd_pgc(struct k8048 *k, int pgd, int pgc)
{
    if(k->flags & FLIPDTR)
        pgd= HIGH-pgd;

    if(k->flags & FLIPRTS)
        pgc= HIGH-pgc;

    set_dtr_rts(k->fd, pgd, pgc);
}

/*
 * Set !MCLR/VPP
 */
void
io_set_mclr(struct k8048 *k, int mclr)
{
    if(k->flags & FLIPTX)
        mclr= HIGH-mclr;

    set_tx(k->fd, mclr);
}

/*
 * Get PGD
 */
unsigned char
io_get_pgd(struct k8048 *k)
{
    unsigned char cts= get_cts(k->fd);

    if(k->flags & FLIPCTS)
        return (cts) ? LOW : HIGH;

    return (cts) ? HIGH : LOW;
}

/******************************************************************************
 * K8048 BIT BANG SHORTCUTS
 *****************************************************************************/

/*
 * Put board in standby
 */
void
io_standby(struct k8048 *k)
{
    io_set_pgd_pgc(k, LOW, LOW);
    io_set_mclr(k, LOW);
    io_cursor('-');
}

/*
 * Initialise board and device for Program/Verify Mode
 */
void
io_init_program_verify(struct k8048 *k)
{
    io_cursor('+');

    io_standby(k);
    usleep(50000);               /* 50ms */

    io_set_mclr(k, HIGH);
    usleep(50000);               /* 50ms */
}

/*
 * prepare data output line for data input
 *
 * on the k8048 the data output line is taken high
 * to pull-up the data line from the PIC on the k8076
 * this is an arbitrary choice, but the best bet is
 * to let the PIC sink the source as we can never
 * know if the data output line will ever be O/C
 */
void
io_data_input_pullup(struct k8048 *k)
{
    if(k->flags & ENABLEPULLUP)
        io_set_pgd(k, HIGH);
    else
        io_set_pgd(k, LOW);
}

/*
 * Clock a bit into Chip (WRITE)
 */
void
io_clock_out(struct k8048 *k, int bit)
{
    io_set_pgd(k, bit);         /* output bit */

    io_set_pgc(k, HIGH);        /* clock high */
    io_sleep(k);

    io_set_pgc(k, LOW);         /* clock low */
    io_sleep(k);
}

/*
 * Clock a bit out of chip into host (READ)
 */
unsigned char
io_clock_in(struct k8048 *k)
{
    unsigned char clkin;

    io_set_pgc(k, HIGH);        /* clock high */
    io_sleep(k);

    io_set_pgc(k, LOW);         /* clock low */
    io_usleep(k);

    clkin= io_get_pgd(k);       /* input bit */

    return clkin;
}

/*
 * Output command word
 */
void
io_command_out(struct k8048 *k, char *command)
{
    int i;
    int l=strlen(command)-1;

    for(i=l; i>=0; i--)
        io_clock_out(k, command[i] - '0');

    io_sleep(k);                /* 1us Tdly1|Tdly2 / P5 20ns */
}

/*
 * Output [START bit:14 bit word out:STOP bit] (pic16f/12f)
 */
void
io_word_out14(struct k8048 *k, unsigned short word)
{
    int i;

    io_cursor('o');

    io_clock_out(k, HIGH);      /* start bit */

    for(i=0; i<14; i++)
        io_clock_out(k, (word >> i) & 1);
    
    io_clock_out(k, HIGH);      /* stop bit */
    
    io_sleep(k);                /* 1us Tdly1|Tdly2 */
}

/*
 * Input [START bit:14 bit word in:STOP bit] (pic16f/12f)
 */
unsigned short
io_word_in14(struct k8048 *k)
{
    int i;
    unsigned short word=0;

    io_cursor('i');

    io_data_input_pullup(k);

    io_clock_in(k);             /* start bit */

    for(i=0; i<14; i++)
        word|= (io_clock_in(k) << i);

    io_clock_in(k);             /* stop bit */
    
    io_sleep(k);                /* 1us Tdly1|Tdly2 */
    
    return word;
}

/*
 * Output [16 bit word out] (pic18f)
 */
void
io_word_out(struct k8048 *k, unsigned short word)
{
    int i;

    io_cursor('o');

    for(i=0; i<16; i++)
        io_clock_out(k, (word >> i) & 1);

    io_sleep(k);                /* P5A 20ns */
}

/*
 * Input [8 bit zero out:8 bit byte in] (pic18f)
 */
unsigned char
io_byte_in(struct k8048 *k)
{
    int i;
    unsigned char byte=0;

    io_cursor('i');

    for(i=0; i<8; i++)
        io_clock_out(k, LOW);

    io_data_input_pullup(k);

    io_sleep(k);                /* P6 20ns */

    for(i=0; i<8; i++)
        byte|= (io_clock_in(k) << i);

    io_sleep(k);                /* P5A 20ns */
    
    return byte;
}

/*
 * receive debug lines (sent by debug assembly code from pic over serial link)
 */
void
io_debug(struct k8048 *k)
{
    int i, j=0;
    unsigned char c, line[STRLEN];

    debug_stop=0;

    signal(SIGINT, io_debug_signal);

    io_data_input_pullup(k);

    while(!debug_stop)
    {
        io_set_pgc(k, LOW);                     /* PGC LOW */
        usleep(DEBUGTIMER);

        while(!io_get_pgd(k) && !debug_stop)    /* WAIT FOR PGD HIGH (flipped) */
            io_usleep(k);
        
        io_set_pgc(k, HIGH);                    /* PGC HIGH */
        usleep(DEBUGTIMER);
        
        while( io_get_pgd(k) && !debug_stop)    /* WAIT FOR PGD LOW (flipped) */
            io_usleep(k);

        c=0;
        for(i=0; i<8; i++)
        {
            io_set_pgc(k, LOW);                 /* PGC LOW */
            usleep(DEBUGTIMER);

            c= c>>1;                            /* CLOCK BIT IN */

            if(!io_get_pgd(k))                  /* (flipped) */
                c|=128;

            io_set_pgc(k, HIGH);                /* PGC HIGH */
            usleep(DEBUGTIMER);
        }
                
        printf("%s: read byte: %02x %c\n", __FUNCTION__, c, (c >= ' ' && c < 128) ? c : '.');

        if(c==0x03)                             /* ETX */
        {
            if(j>1 && line[0]==0x02)            /* STX */
            {
                line[j]='\0';
                printf("%s: read line: %s", __FUNCTION__, &line[1]);
            }
            j=0;
        }
        else if(j<STRMAX)
            line[j++]=c;

        usleep(DEBUGTIMER);
    }
    printf("\n");

    signal(SIGINT, SIG_DFL);
}

/*
 * Signal handler for above
 */
void
io_debug_signal()
{
    debug_stop= 1;
}

/*
 * set lines to specific levels for testing purposes
 *
 * output:
 *
 * Tx  SET  (+VE) (D-SUB 3) [1 seconds] CTS IN: 32
 * Tx  CLR  (-VE) (D-SUB 3) [1 seconds] CTS IN: 32
 *
 * DTR SET  (+VE) (D-SUB 4) [1 seconds] CTS IN: 32
 * DTR CLR  (-VE) (D-SUB 4) [1 seconds] CTS IN: 0
 *
 * RTS SET  (+VE) (D-SUB 7) [1 seconds] CTS IN: 0
 * RTS CLR  (-VE) (D-SUB 7) [1 seconds] CTS IN: 0
 *
 * VPP LOW  (0V)  (ICSP 1)  [1 seconds] PGD IN: 0
 * VPP HIGH (12V) (ICSP 1)  [1 seconds] PGD IN: 0
 *
 * PGD LOW  (0V)  (ICSP 4)  [1 seconds] PGD IN: 0
 * PGD HIGH (5V)  (ICSP 4)  [1 seconds] PGD IN: 1
 *
 * PGC LOW  (0V)  (ICSP 5)  [1 seconds] PGD IN: 1
 * PGC HIGH (5V)  (ICSP 5)  [1 seconds] PGD IN: 1
 *
 * the above is the output of a working k8048 and the voltages
 * measured on the ICSP pins match those expected by the
 * diagnostic prompt.
 */
void
io_test1(struct k8048 *k, int t)
{
    printf("\nTEST MODE\n\n");

    /* RS-232 */

    printf("Tx  SET  (+VE) (D-SUB 3) [%d seconds] ", t); fflush(stdout);
    set_tx(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("Tx  CLR  (-VE) (D-SUB 3) [%d seconds] ", t); fflush(stdout);
    set_tx(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));

    printf("DTR SET  (+VE) (D-SUB 4) [%d seconds] ", t); fflush(stdout);
    set_dtr(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("DTR CLR  (-VE) (D-SUB 4) [%d seconds] ", t); fflush(stdout);
    set_dtr(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));
    
    printf("RTS SET  (+VE) (D-SUB 7) [%d seconds] ", t); fflush(stdout);
    set_rts(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("RTS CLR  (-VE) (D-SUB 7) [%d seconds] ", t); fflush(stdout);
    set_rts(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));

    /* ICSP */

    io_set_pgd(k, LOW);
    io_set_pgc(k, LOW);

    printf("VPP LOW  (0V)  (ICSP 1)  [%d seconds] ", t); fflush(stdout);
    io_set_mclr(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("VPP HIGH (12V) (ICSP 1)  [%d seconds] ", t); fflush(stdout);
    io_set_mclr(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("PGD LOW  (0V)  (ICSP 4)  [%d seconds] ", t); fflush(stdout);
    io_set_pgd(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("PGD HIGH (5V)  (ICSP 4)  [%d seconds] ", t); fflush(stdout);
    io_set_pgd(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("PGC LOW  (0V)  (ICSP 5)  [%d seconds] ", t); fflush(stdout);
    io_set_pgc(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("PGC HIGH (5V)  (ICSP 5)  [%d seconds] ", t); fflush(stdout);
    io_set_pgc(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("TEST DONE\n\n");
    
    io_set_mclr(k, LOW);
    io_set_pgd(k, LOW);
    io_set_pgc(k, LOW);
}

void
io_test2(struct k8048 *k, int test)
{
    int clk=LOW;

    printf("\nTEST MODE 2: TOGGLE PGC (CTRL-C TO EXIT)\n\n");

    io_set_mclr(k, LOW);
    io_set_pgd(k, LOW);

    /*
     * MARK time (Linux) AMD64 3800+
     * =============================
     *
     * ttyS0
     * -----
     *
     *  There is a 1.5uS rise time for each of the following.
     *
     *      io_sleep:  3.44uS 
     *      io_usleep: 5.56uS
     *      none:      2.39uS (probably unreliable for timing purposes)
     *
     * ttyUSB1
     * -------
     *
     *  This is dependant upon usb_control_msg() which always takes a minimum of 3ms.
     *
     *      io_sleep:  3ms
     *      io_usleep: 3ms
     *      none:      3ms
     */
    while(1)
    {
        io_set_pgc(k, clk);
        switch(test)
        {
            case  0:    break;

            case -1:    io_sleep(k);
                        break;

            case -2:    io_usleep(k);
                        break;
        }
        clk= HIGH-clk;
    }

    io_set_pgc(k, LOW);
}

/*
 * i/o busy cursor
 */
#define DELAY (3)
void
io_cursor(char c)
{
	static int c_count = 0;
	static const char arrow[] = "|/-\\";
	int len = strlen(arrow);

	if(++c_count % (DELAY * len - 1) == 0)
	{
		fprintf(stdout,"%c[%c]\r", arrow[ len - ( c_count % len ) - 1 ], c);
		fflush(stdout);
	}
}

==> k8048-1.04.tar.gz: k8048/k8048/io.c <==
/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2007 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: io.c,v 1.53 2008/09/19 06:39:02 darron Exp $
 *
 * The Velleman programmer board has the following serial interface
 * characteristics (d-sub 9):
 *
 * out:
 * 
 * DSUB-9                       ICSP
 * 7        RTS -> PGC          5
 * 4        DTR -> PGD          4
 * 3        TX  -> !MCLR/VPP    1
 * 
 * in:
 *
 * 8        CTS <- PGD          4
 *
 * other:
 *
 * 9<->3    RI <- TX
 *
 * Although the interface connects to the serial port, it doesn't
 * utilise a standard serial protocol and I/O is performed using
 * the "bit banging" method.
 *
 * Data output bits are sent through the DTR line and clocked on the falling
 * edge of RTS.
 * 
 * The Tx line is utilised to enable the application of the high
 * programming voltage on !MCLR/VPP and not for sending data bits.
 *
 * Data input bits are read from the CTS line using the same clock i/f as
 * above with the PGD output taken high for correct circuit operation.
 *
 * D-SUB9
 *    1 2 3 4 5
 *     6 7 8 9
 *    1:CD
 *    2:Rx
 *    3:Tx
 *    4:DTR
 *    5:GND
 *    6:DSR
 *    7:RTS
 *    8:CTS
 *    9:RI
 *
 *
 * ICSP
 *    5 4 3 2 1
 *    5:PGC
 *    4:PGD
 *    3:VSS/GND
 *    2:VDD (+5)
 *    1:VPP (+12)
 *
 *
 * SERIAL PORT VOLTAGES
 *  
 * TEST 1
 * ------
 *  switch set to RUN, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 5V
 *       2 5V
 *       3 0V
 *       4 5V
 *       5 5V
 *
 * TEST 2
 * ------
 *  switch set to PROG, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 12V
 *       2 5V
 *       3 0V
 *       4 5V
 *       5 5V
 *
 * TEST 3
 * ------
 *  switch set to STANDBY, FreeBSD 6.1, no chip installed, no port setup
 *  ICSP 1 0V
 *       2 0V
 *       3 0V
 *       4 5V
 *       5 5V
 */

#include "k8048.h"

#define USLEEP(n) { if(n>0) usleep(n); }

/******************************************************************************
 * debug CTRL-C detection
 *****************************************************************************/
int debug_stop=0;

/******************************************************************************
 * INIT/CLOSE I/O
 *****************************************************************************/
int
io_open(struct k8048 *k)
{
    int fd;
    
    if( (fd= open_port(k->device)) >= 0)
        io_standby(k);

    return fd;
}

void
io_close(struct k8048 *k)
{
    io_standby(k);
    close(k->fd);
}

/******************************************************************************
 * K8048 BIT BANGING NB. RS232 output is positive when feature enabled
 *****************************************************************************/

/*
 * Set PGD (DATA)
 */
void
io_set_pgd(struct k8048 *k, int pgd)
{
    if(k->flags & FLIPDTR)      /* inverting buffer present on hardware (k8048) */
        pgd= HIGH-pgd;

    set_dtr(k->fd, pgd);        /* non inverting buffer present on hardware (k8076 *) */
                                /* * in this case the accumulation of inverters is non inverting */
}

/*
 * Set PGC (CLOCK)
 */
void
io_set_pgc(struct k8048 *k, int pgc)
{
    if(k->flags & FLIPRTS)
        pgc= HIGH-pgc;

    set_rts(k->fd, pgc);
}

/*
 * Set PGD/PGC (DATA/CLOCK) 
 */
void
io_set_pgd_pgc(struct k8048 *k, int pgd, int pgc)
{
    if(k->flags & FLIPDTR)
        pgd= HIGH-pgd;

    if(k->flags & FLIPRTS)
        pgc= HIGH-pgc;

    set_dtr_rts(k->fd, pgd, pgc);
}

/*
 * Set !MCLR/VPP
 */
void
io_set_mclr(struct k8048 *k, int mclr)
{
    if(k->flags & FLIPTX)
        mclr= HIGH-mclr;

    set_tx(k->fd, mclr);
}

/*
 * Get PGD
 */
unsigned char
io_get_pgd(struct k8048 *k)
{
    unsigned char cts= get_cts(k->fd);

    if(k->flags & FLIPCTS)
        return (cts) ? LOW : HIGH;

    return (cts) ? HIGH : LOW;
}

/******************************************************************************
 * K8048 BIT BANG SHORTCUTS
 *****************************************************************************/

/*
 * Put board in standby
 */
void
io_standby(struct k8048 *k)
{
    io_set_pgd_pgc(k, LOW, LOW);
    io_set_mclr(k, LOW);
    io_cursor('-');
}

/*
 * Initialise board and device for Program/Verify Mode
 */
void
io_init_program_verify(struct k8048 *k)
{
    io_cursor('+');

    io_standby(k);
    usleep(50000);               /* 50ms */

    io_set_mclr(k, HIGH);
    usleep(50000);               /* 50ms */
}

/*
 * prepare data output line for data input
 *
 * on the k8048 the data output line is taken high
 * to pull-up the data line from the PIC on the k8076
 * this is an arbitrary choice, but the best bet is
 * to let the PIC sink the source as we can never
 * know if the data output line will ever be O/C
 */
void
io_data_input_pullup(struct k8048 *k)
{
    if(k->flags & ENABLEPULLUP)
        io_set_pgd(k, HIGH);
    else
        io_set_pgd(k, LOW);
}

/*
 * Clock a bit into Chip (WRITE)
 */
void
io_clock_out(struct k8048 *k, int bit)
{
    io_set_pgd(k, bit);         /* output bit */

    io_set_pgc(k, HIGH);        /* clock high */
    USLEEP(k->sleep_short);

    io_set_pgc(k, LOW);         /* clock low */
    USLEEP(k->sleep_short);
}

/*
 * Clock a bit out of chip into host (READ)
 */
unsigned char
io_clock_in(struct k8048 *k)
{
    unsigned char clkin;

    io_set_pgc(k, HIGH);        /* clock high */
    USLEEP(k->sleep_short);

    io_set_pgc(k, LOW);         /* clock low */
    USLEEP(k->sleep_long);      /* XXX */

    clkin= io_get_pgd(k);       /* input bit */

    return clkin;
}

/*
 * Output command word
 */
void
io_command_out(struct k8048 *k, char *command)
{
    int i;
    int l=strlen(command)-1;

    for(i=l; i>=0; i--)
        io_clock_out(k, command[i] - '0');

    USLEEP(k->sleep_short);         /* 1us Tdly1|Tdly2 / P5 20ns */
}

/*
 * Output [START bit:14 bit word out:STOP bit] (pic16f/12f)
 */
void
io_word_out14(struct k8048 *k, unsigned short word)
{
    int i;

    io_cursor('o');

    io_clock_out(k, HIGH);      /* start bit */

    for(i=0; i<14; i++)
        io_clock_out(k, (word >> i) & 1);
    
    io_clock_out(k, HIGH);      /* stop bit */
    
    USLEEP(k->sleep_short);         /* 1us Tdly1|Tdly2 */
}

/*
 * Input [START bit:14 bit word in:STOP bit] (pic16f/12f)
 */
unsigned short
io_word_in14(struct k8048 *k)
{
    int i;
    unsigned short word=0;

    io_cursor('i');

    io_data_input_pullup(k);

    io_clock_in(k);             /* start bit */

    for(i=0; i<14; i++)
        word|= (io_clock_in(k) << i);

    io_clock_in(k);             /* stop bit */
    
    USLEEP(k->sleep_short);         /* 1us Tdly1|Tdly2 */
    
    return word;
}

/*
 * Output [16 bit word out] (pic18f)
 */
void
io_word_out(struct k8048 *k, unsigned short word)
{
    int i;

    io_cursor('o');

    for(i=0; i<16; i++)
        io_clock_out(k, (word >> i) & 1);

    USLEEP(k->sleep_short);         /* P5A 20ns */
}

/*
 * Input [8 bit zero out:8 bit byte in] (pic18f)
 */
unsigned char
io_byte_in(struct k8048 *k)
{
    int i;
    unsigned char byte=0;

    io_cursor('i');

    for(i=0; i<8; i++)
        io_clock_out(k, LOW);

    io_data_input_pullup(k);

    USLEEP(k->sleep_short);         /* P6 20ns */

    for(i=0; i<8; i++)
        byte|= (io_clock_in(k) << i);

    USLEEP(k->sleep_short);         /* P5A 20ns */
    
    return byte;
}

/*
 * receive debug lines (sent by debug assembly code from pic over serial link)
 */
void
io_debug(struct k8048 *k)
{
    int i, j=0;
    unsigned char c, line[STRLEN];

    debug_stop=0;

    signal(SIGINT, io_debug_signal);

    io_data_input_pullup(k);

    while(!debug_stop)
    {
        io_set_pgc(k, LOW);                     /* PGC LOW */
        usleep(DEBUGTIMER);

        while(!io_get_pgd(k) && !debug_stop)    /* WAIT FOR PGD HIGH (flipped) */
            usleep(1);
        
        io_set_pgc(k, HIGH);                    /* PGC HIGH */
        usleep(DEBUGTIMER);
        
        while( io_get_pgd(k) && !debug_stop)    /* WAIT FOR PGD LOW (flipped) */
            usleep(1);

        c=0;
        for(i=0; i<8; i++)
        {
            io_set_pgc(k, LOW);                 /* PGC LOW */
            usleep(DEBUGTIMER);

            c= c>>1;                            /* CLOCK BIT IN */

            if(!io_get_pgd(k))                  /* (flipped) */
                c|=128;

            io_set_pgc(k, HIGH);                /* PGC HIGH */
            usleep(DEBUGTIMER);
        }
                
        printf("%s: read byte: %02x %c\n", __FUNCTION__, c, (c >= ' ' && c < 128) ? c : '.');

        if(c==0x03)                             /* ETX */
        {
            if(j>1 && line[0]==0x02)            /* STX */
            {
                line[j]='\0';
                printf("%s: read line: %s", __FUNCTION__, &line[1]);
            }
            j=0;
        }
        else if(j<STRMAX)
            line[j++]=c;

        usleep(DEBUGTIMER);
    }
    printf("\n");

    signal(SIGINT, SIG_DFL);
}

/*
 * Signal handler for above
 */
void
io_debug_signal()
{
    debug_stop= 1;
}

/*
 * set lines to specific levels for testing purposes
 *
 * output:
 *
 * Tx  SET  (+VE) (D-SUB 3) [1 seconds] CTS IN: 32
 * Tx  CLR  (-VE) (D-SUB 3) [1 seconds] CTS IN: 32
 *
 * DTR SET  (+VE) (D-SUB 4) [1 seconds] CTS IN: 32
 * DTR CLR  (-VE) (D-SUB 4) [1 seconds] CTS IN: 0
 *
 * RTS SET  (+VE) (D-SUB 7) [1 seconds] CTS IN: 0
 * RTS CLR  (-VE) (D-SUB 7) [1 seconds] CTS IN: 0
 *
 * VPP LOW  (0V)  (ICSP 1)  [1 seconds] PGD IN: 0
 * VPP HIGH (12V) (ICSP 1)  [1 seconds] PGD IN: 0
 *
 * PGD LOW  (0V)  (ICSP 4)  [1 seconds] PGD IN: 0
 * PGD HIGH (5V)  (ICSP 4)  [1 seconds] PGD IN: 1
 *
 * PGC LOW  (0V)  (ICSP 5)  [1 seconds] PGD IN: 1
 * PGC HIGH (5V)  (ICSP 5)  [1 seconds] PGD IN: 1
 *
 * the above is the output of a working k8048 and the voltages
 * measured on the ICSP pins match those expected by the
 * diagnostic prompt.
 */
void
io_test1(struct k8048 *k, int t)
{
    printf("\nTEST MODE\n\n");

    /* RS-232 */

    printf("Tx  SET  (+VE) (D-SUB 3) [%d seconds] ", t); fflush(stdout);
    set_tx(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("Tx  CLR  (-VE) (D-SUB 3) [%d seconds] ", t); fflush(stdout);
    set_tx(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));

    printf("DTR SET  (+VE) (D-SUB 4) [%d seconds] ", t); fflush(stdout);
    set_dtr(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("DTR CLR  (-VE) (D-SUB 4) [%d seconds] ", t); fflush(stdout);
    set_dtr(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));
    
    printf("RTS SET  (+VE) (D-SUB 7) [%d seconds] ", t); fflush(stdout);
    set_rts(k->fd, HIGH);
    sleep(t);
    printf("CTS IN: %d\n", get_cts(k->fd));
    printf("RTS CLR  (-VE) (D-SUB 7) [%d seconds] ", t); fflush(stdout);
    set_rts(k->fd, LOW);
    sleep(t);
    printf("CTS IN: %d\n\n", get_cts(k->fd));

    /* ICSP */

    io_set_pgd(k, LOW);
    io_set_pgc(k, LOW);

    printf("VPP LOW  (0V)  (ICSP 1)  [%d seconds] ", t); fflush(stdout);
    io_set_mclr(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("VPP HIGH (12V) (ICSP 1)  [%d seconds] ", t); fflush(stdout);
    io_set_mclr(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("PGD LOW  (0V)  (ICSP 4)  [%d seconds] ", t); fflush(stdout);
    io_set_pgd(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("PGD HIGH (5V)  (ICSP 4)  [%d seconds] ", t); fflush(stdout);
    io_set_pgd(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("PGC LOW  (0V)  (ICSP 5)  [%d seconds] ", t); fflush(stdout);
    io_set_pgc(k, LOW);
    sleep(t);
    printf("PGD IN: %d\n",   io_get_pgd(k));
    printf("PGC HIGH (5V)  (ICSP 5)  [%d seconds] ", t); fflush(stdout);
    io_set_pgc(k, HIGH);
    sleep(t);
    printf("PGD IN: %d\n\n", io_get_pgd(k));

    printf("TEST DONE\n\n");
    
    io_set_mclr(k, LOW);
    io_set_pgd(k, LOW);
    io_set_pgc(k, LOW);
}

void
io_test2(struct k8048 *k, int test)
{
    printf("error: test2 unimplemented\n");
    exit(-1);
#if 0
    int clk=LOW;

    printf("\nTEST MODE 2: TOGGLE PGC (CTRL-C TO EXIT)\n\n");

    io_set_mclr(k, LOW);
    io_set_pgd(k, LOW);

    /*
     * MARK time (Linux) AMD64 3800+
     * =============================
     *
     * ttyS0
     * -----
     *
     *  There is a 1.5uS rise time for each of the following.
     *
     *      io_sleep:  3.44uS 
     *      io_usleep: 5.56uS
     *      none:      2.39uS (probably unreliable for timing purposes)
     *
     * ttyUSB1
     * -------
     *
     *  This is dependant upon usb_control_msg() which always takes a minimum of 3ms.
     *
     *      io_sleep:  3ms
     *      io_usleep: 3ms
     *      none:      3ms
     */
    while(1)
    {
        io_set_pgc(k, clk);
        switch(test)
        {
            case  0:    break;

            case -1:    io_sleep(k);
                        break;

            case -2:    io_sleep(k);
                        break;
        }
        clk= HIGH-clk;
    }

    io_set_pgc(k, LOW);
#endif
}

/*
 * i/o busy cursor
 */
#define DELAY (2)
void
io_cursor(char c)
{
	static int c_count = 0;
	static const char arrow[] = "|/-\\";
	int len = strlen(arrow);

	if(++c_count % (DELAY * len - 1) == 0)
	{
        fprintf(stdout,"%c[%c]\r", arrow[ len - ( c_count % len ) - 1 ], c);
		fflush(stdout);
	}
}
/*
 * vim: shiftwidth=4 expandtab tabstop=4 softtabstop=4
 */
